/*
 *  Copyright (c) 2010-2013, Rutgers, The State University of New Jersey
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above copyright
 *        notice, this list of conditions and the following disclaimer in the
 *        documentation and/or other materials provided with the distribution.
 *      * Neither the name of the organization(s) stated above nor the
 *        names of its contributors may be used to endorse or promote products
 *        derived from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
 *  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * @file   mfaggregator.cpp
 * @author wontoniii (bronzino@winlab.rutgers.edu)
 * @date   July, 2011
 * @brief  Temp.
 *
 * Temp
 */

#include <map>

#include <include/mfhopmsg.hh>

#include "mftypes.h"
#include "mfchunkinfo.h"
#include "mfdeviceinfo.h"
#include "mfetherproto.h"
#include "mfipproto.h"
#include "mfbuffer.h"
#include "mfeventqueue.h"
#include "mftypes.h"
#include "mflog.h"
#include "mfaggregator.h"
#include "mfpacketsupport.h"

MF_Aggregator::MF_Aggregator() : mPool(10) {
	lastHopID = 0;
}

MF_Aggregator::MF_Aggregator(MF_EventQueue *eq) : mPool(10) {
	this->eq = eq;
	lastHopID = 0;
}

MF_Aggregator::MF_Aggregator(MF_EventQueue *eq, int bufSize) : mPool(bufSize) {
	this->eq = eq;
	lastHopID = 0;
}

MF_Aggregator::~MF_Aggregator() {
	//Clean buffer
	mPool.dePopulate();
}


/*
 * a data copy happens here
 */
void MF_Aggregator::handleData(const u_char *packet) {
	hop_data_t *hop = (hop_data_t *)packet;
	//MF_Log::mf_log(MF_DEBUG, "MF_Aggregator:handleData Recvd data for chunk = %u seqNum=%u", ntohl(hop->hop_ID), ntohl(hop->seq_num));
	map <u_int, MF_ChunkInfo *>::iterator it;
	it = outsChunks.find(ntohl(hop->hop_ID));
	if(it==outsChunks.end()){
		MF_Log::mf_log(MF_ERROR, "MF_Aggregator:handleData data received for chunk we are not expecting");
		return;
	}
	else{
		MF_ChunkInfo *ci = it->second;
		vector <u_char *> *pList = ci->getPacketList();
		if(ci) memcpy(((*pList)[ntohl(hop->seq_num)]+ETH_HEADER_SIZE+IP_HEADER_SIZE), packet, HOP_HEADER_SIZE + MF_PacketSupport::MAX_PAYLOAD_SIZE);
	}
}

int MF_Aggregator::handleCsyn(u_int hopID, u_int pktCount) {
	MF_Log::mf_log(MF_INFO, "MF_Aggregator:handleCsyn Recvd CSYN hop_ID = %d, pkt_count = %d",hopID, pktCount);
	MF_ChunkInfo *ci;
	u_int prevHopID;
	if (lastHopID < hopID || hopID == 0) {
		ci = new MF_ChunkInfo();
		vector<u_char *> *chkpkt = ci->getPacketList();
		/* late reset */
		ci->reset();
		if (mPool.getVectorBySlots(chkpkt, pktCount) < 0) {
			MF_Log::mf_log(MF_WARN, "MF_Aggregator:handleCsyn Run out of buffer space...");
			return -1; //no ack send out, sender will timeout
		}
		ci->putHopID(hopID);
		prevHopID = lastHopID;
		lastHopID = hopID;
		tryRelease(prevHopID);
		ci->putChunkPktCnt(pktCount);
		outsChunks.insert(make_pair(hopID, ci));
		return 1;
	} else if (lastHopID == hopID){
		map <u_int, MF_ChunkInfo *>::iterator it = outsChunks.find(hopID);
		if(it!=outsChunks.end()){
			if(it->second->isDelivered() == false){
				return 1;
			}
			else if (it->second->isDelivered() == true) {//ack got lost
				return 2;
			}
		}
		else{
			MF_Log::mf_log(MF_DEBUG, "MF_Aggregator:handleCsyn chunk was already delivered so it has been deleted already");
			return 2;
		}
	}
	else if ((lastHopID > hopID)) {//this should not happen
		MF_Log::mf_log(MF_DEBUG, "MF_Aggregator:handleCsyn Recvd CSYN for OLD chunk. hop_ID = %d", hopID);
	}
	return 0;
}

void MF_Aggregator::tryRelease(u_int chkID){
	MF_Log::mf_log(MF_DEBUG, "MF_Aggregator:tryRelease releasing chunk read from upper layers with chk ID %u", chkID);
	map <u_int, MF_ChunkInfo *>::iterator it;
	it = outsChunks.find(chkID);
	if(it!=outsChunks.end()){
		MF_ChunkInfo *ci = it->second;
		tryRelease(ci);
	}
}

void MF_Aggregator::tryRelease(MF_ChunkInfo *ci){
	MF_Log::mf_log(MF_DEBUG, "MF_Aggregator:tryRelease releasing chunk read from upper layers with chk ID %u", ci->getHopID());
	if(lastHopID == ci->getHopID() || !ci->isReleased()){
		MF_Log::mf_log(MF_DEBUG, "MF_Aggregator:tryRelease I have not received a new csyn so I'll wait just in case");
	}
	else{
		MF_Log::mf_log(MF_DEBUG, "MF_Aggregator:tryRelease I have received a new csyn so I'll delete");
		mPool.putVector(ci->getPacketList());
		outsChunks.erase(ci->getHopID());
		delete ci;
	}
}

void MF_Aggregator::releaseChunk(u_int chkID){
	MF_Log::mf_log(MF_DEBUG, "MF_Aggregator:releaseChunk releasing chunk read from upper layers with chk ID %u", chkID);
	map <u_int, MF_ChunkInfo *>::iterator it;
	it = outsChunks.find(chkID);
	if(it!=outsChunks.end()){
		MF_ChunkInfo *ci = it->second;
		ci->setReleased(true);
		tryRelease(ci);
	}
}

MF_ChunkInfo *MF_Aggregator::getChunkInfo(u_int hopID){
	MF_Log::mf_log(MF_DEBUG, "MF_Aggregator:getChunkInfo");
	map <u_int, MF_ChunkInfo *>::iterator it;
	it = outsChunks.find(hopID);
	if(it==outsChunks.end()){
		MF_Log::mf_log(MF_DEBUG, "MF_Aggregator:getChunkInfo chunk %u not found", hopID);
		return NULL;
	}
	else{
		return it->second;
	}
}
