/*
 *  Copyright (c) 2010-2013, Rutgers, The State University of New Jersey
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above copyright
 *        notice, this list of conditions and the following disclaimer in the
 *        documentation and/or other materials provided with the distribution.
 *      * Neither the name of the organization(s) stated above nor the
 *        names of its contributors may be used to endorse or promote products
 *        derived from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
 *  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * @file   mfsegmentor.h
 * @author wontoniii (bronzino@winlab.rutgers.edu), czhang
 * @date   July, 2011
 * @brief  Class that handles the transmission of data chunks, csyn and assoc/dassoc packets.
 *
 * Class that handles the transmission of data chunks, csyn and assoc/dassoc packets.
 */

#ifndef MF_SEGMENTOR_H_
#define MF_SEGMENTOR_H_

#include <pcap.h>
#include <vector>

#include "mftypes.h"
#include "mfdeviceinfo.h"
#include "mfchunkinfo.h"
#include "mftimer.h"
#include "mfeventqueue.h"

class MF_Timer;

class MF_Segmentor{
	volatile bool mIsCsynSent;
	volatile u_int mCsynSeqNum;
	MF_ChunkInfo *sendingChunk;
	u_int mLastHopID;
	MF_Timer mTimer;
	vector<u_int> mNonAcked;
	MF_EventQueue *eq;
	pcap_t *mPcapHandle;
	
	const static u_int MAX_HOP_ID = 65535;
	const static u_int MAX_CSYN_SEQ_NUM = 65535;
	
public:
	MF_Segmentor();
	MF_Segmentor(MF_EventQueue *eq, pcap_t *handle, int devId, int t);
	~MF_Segmentor();
	void setPCAPHandle(pcap_t *);
	u_int getNewHopID();
	void resetHopID(u_int);
	u_int updateCsynSeqNum();
	u_int getCsynSeqNum();
	void sendCsyn(MF_ChunkInfo *, MF_DeviceInfo *);
	void csynTO(u_int chunkID, MF_DeviceInfo *);
	void sendData(MF_ChunkInfo *, MF_DeviceInfo *);
	bool isAllSent(u_char *);
	bool isSending();
	void procSend(MF_ChunkInfo *, MF_DeviceInfo *);
	void fillHeader(MF_ChunkInfo *, MF_DeviceInfo *);
	void fillHopHeader(u_char *, u_int, u_int, u_int);
	void sendAssoRequest(u_int, u_int, u_short, MF_DeviceInfo *);
	void sendDassoRequest(u_int, u_int, MF_DeviceInfo *);
	void handleAck(u_int hop_ID, u_int pkt_count, u_char *bitmap, MF_DeviceInfo *);
	bool sendAck(MF_ChunkInfo *ci, MF_DeviceInfo *di);
	void sendLPAck(u_int, u_int, u_int, MF_DeviceInfo *);
};

#endif /* MF_SEGMENTOR_H_ */
